//
//  YHOOSManager.swift
//  galaxy
//
//  Created by EDY on 2024/1/16.
//  Copyright © 2024 https://www.galaxy-immi.com. All rights reserved.
//

import UIKit
import AliyunOSSiOS

public protocol YHError {
    var _domain: String { get }
    var _code: Int { get }
    var _userInfo: AnyObject? { get }
}

extension NSError : YHError {
    @nonobjc
    public var _domain: String { return domain }
    
    @nonobjc
    public var _code: Int { return code }
    
    @nonobjc
    public var _userInfo: AnyObject? { return userInfo as NSDictionary }
}

class YHOSSModel: YHBaseModel {
    var bucket: String = ""
    var Expiration: String = ""
    var bucketPath: String = ""
    var accessKeyId: String = ""
    var accessKeySecret: String = ""
    var endpoint: String = ""
    var securityToken: String = ""
}

class YHOSSManager: NSObject, URLSessionDelegate {
    static let share = YHOSSManager()
    var model: YHOSSModel = YHOSSModel()
    var mProvider: OSSStsTokenCredentialProvider!
    var mClient: OSSClient!
    var fileName: [String] = []
    
    override init() {
    }
    
    func getOSSID() {
        let strUrl = YHBaseUrlManager.shared.curOssURL() + YHAllApiName.OSS.getID
        let _ = YHNetRequest.getRequest(url: strUrl) { [weak self] json, code in
            //1. json字符串 转 对象
            guard let self = self else { return }
            let dic = json.data
            guard let resultModel = YHOSSModel.deserialize(from: dic as? Dictionary) else {
                return
            }
            self.model = resultModel
            self.mProvider = OSSStsTokenCredentialProvider(accessKeyId: self.model.accessKeyId, secretKeyId: self.model.accessKeySecret, securityToken: self.model.securityToken)
            self.mClient = OSSClient(endpoint: self.model.endpoint, credentialProvider: self.mProvider)
        } failBlock: { err in
        }
    }
    
    func putObject(image: UIImage, callBackBlock:@escaping (_ success: String?, _ error:YHErrorModel?)->()) {
        let request = OSSPutObjectRequest()
        let size = Int(5 * 1024 * 1024)
        guard let imageData = image.compressOriginalImage(size) else { return }
        request.uploadingData = imageData
        request.bucketName = self.model.bucket
        let timestamp = Date().timeIntervalSince1970
        let randomInt = Int.random(in: 1...100) // 随机生成1到10之间的整数
        let name = "\(UInt64(timestamp))" + "\(randomInt)" + ".jpg"
        self.fileName.append(name)
        request.objectKey = self.model.bucketPath + "/" + "\(name)"
        request.uploadProgress = { (bytesSent: Int64, totalBytesSent: Int64, totalBytesExpectedToSend: Int64) -> Void in
            printLog("bytesSent:\(bytesSent),totalBytesSent:\(totalBytesSent),totalBytesExpectedToSend:\(totalBytesExpectedToSend)")
        }
        
        let task = self.mClient.putObject(request)
        
        task.continue({ (t) -> Any? in
            let result = t
            if (result.error != nil) {
                callBackBlock(nil, YHErrorModel())
            } else {
                let string = "https://" + self.model.bucket + "." + "oss-cn-shenzhen.aliyuncs.com/" + self.model.bucketPath + "/" + "\(self.fileName.first ?? "")"
                self.fileName.removeFirst()
                print("------------\(string)")
                callBackBlock(string, YHErrorModel())
            }
            return
        }).waitUntilFinished()
    }
    
    func putFile(file: String, callBackBlock:@escaping (_ success: String?, _ error:YHErrorModel?)->()) {
        let request = OSSPutObjectRequest()
        guard let fileData = try? Data(contentsOf: URL(string: file) ?? URL(fileURLWithPath: "")) else { return }
        let fileExtension = URL(fileURLWithPath: file).pathExtension.lowercased()
        request.uploadingData = fileData
        request.bucketName = self.model.bucket
        let timestamp = Date().timeIntervalSince1970
        let name = "\(UInt64(timestamp))" + ".\(fileExtension)"
        self.fileName.append(name)
        request.objectKey = self.model.bucketPath + "/" + "\(name)"
        request.uploadProgress = { (bytesSent: Int64, totalBytesSent: Int64, totalBytesExpectedToSend: Int64) -> Void in
            printLog("bytesSent:\(bytesSent),totalBytesSent:\(totalBytesSent),totalBytesExpectedToSend:\(totalBytesExpectedToSend)")
        }
        
        let task = self.mClient.putObject(request)
        
        task.continue({ (t) -> Any? in
            let result = t
            if (result.error != nil) {
                let error: NSError = (task.error)! as NSError
                let result = error.description
                callBackBlock(nil, YHErrorModel())
            } else {
                let string = "https://" + self.model.bucket + "." + "oss-cn-shenzhen.aliyuncs.com/" + self.model.bucketPath + "/" + "\(self.fileName.first ?? "")"
                self.fileName.removeFirst()
                callBackBlock(string, YHErrorModel())
            }
            return
        }).waitUntilFinished()
    }
}
